package krbzsq.surge;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import krbzsq.surge.net.ChipHandler;
import krbzsq.surge.net.HttpRequestDecoder;
import krbzsq.surge.net.HttpResponseEncoder;
import krbzsq.surge.util.Cookies;
import krbzsq.surge.util.Routes;
import krbzsq.surge.view.Page;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

public class Surge implements Runnable {
    private int port;
    private Routes routes;
    private Cookies cookies;
    private Surge surge = this;
    private Page error;
    private ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(Runtime.getRuntime().availableProcessors());

    public Surge() {
        this(80);
    }

    public Surge(int port) {
        this.port = port;
        routes = new Routes(this);
        cookies = new Cookies();
    }

    public Routes routes() {
        return routes;
    }

    public Cookies cookies() {
        return cookies;
    }

    public void setErrorPage(Page page) {
        error = page;
    }

    public Page getErrorPage() {
        return error;
    }

    private ChannelInitializer<SocketChannel> initializer() {
        return new ChannelInitializer<SocketChannel>() {
            @Override
            protected void initChannel(SocketChannel c) throws Exception {
                ChannelPipeline pipeline = c.pipeline();
                pipeline.addLast("decoder", new HttpRequestDecoder());
                pipeline.addLast("encoder", new HttpResponseEncoder());
                pipeline.addLast("handler", new ChipHandler(surge));
            }
        };
    }

    @Override
    public void run() {
        try {
            final ServerBootstrap b = new ServerBootstrap();
            b.group(new NioEventLoopGroup(), new NioEventLoopGroup())
                    .channel(NioServerSocketChannel.class).childHandler(initializer());
            ChannelFuture f = b.bind(port);
            System.out.println("Surge ready on port " + port);
            f.sync().channel().closeFuture().sync();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void schedule(Runnable task, long delay, TimeUnit unit, boolean repeat) {
       if(repeat) {
           scheduler.scheduleAtFixedRate(task, 0, delay, unit);
       } else {
           scheduler.schedule(task, delay, unit);
       }
    }
}
