package krbzsq.surge.util;

import krbzsq.surge.Surge;
import krbzsq.surge.view.Page;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

public class Routes {
    private Map<String, Route> routes = new HashMap<>();
    private Surge surge;

    public Routes(Surge surge) {
        this.surge = surge;
    }

    public void loadXML(final String file, boolean enableReload) {
        try {
            final Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new File(file));
            doc.normalizeDocument();
            NodeList list = doc.getElementsByTagName("route");
            for(int i = 0; i < list.getLength(); i++) {
                Element e = (Element) list.item(i);
                String path = e.getAttribute("path");
                Class<?> page = Class.forName(e.getAttribute("page"));
                String method = e.getAttribute("method");
                int rules = 0;
                if(method == null) {
                    rules |= Route.GET;
                } else {
                    switch (method) {
                        case "GET":
                            rules |= Route.GET;
                            break;
                        case "POST":
                            rules |= Route.POST;
                            break;
                    }
                }
                routes.put(path, new Route(path, (Page) page.newInstance(), rules));
            }
            System.out.println("Loaded " + routes.size() + " routes.");
            if(enableReload) {
                surge.schedule(new Runnable() {
                    @Override
                    public void run() {
                        loadXML(file, false);
                    }
                }, 1, TimeUnit.SECONDS, true);
            }
        } catch (SAXException | IOException | ParserConfigurationException |
                ClassNotFoundException | InstantiationException | IllegalAccessException e) {
            e.printStackTrace();
        }
    }

    public void add(String s, Page page, int... rules) {
        routes.put(s, new Route(s, page, rules));
    }

    public void add(String s, Class<? extends Page> c, int... rules) {
        try {
            routes.put(s,new Route(s,c.newInstance(),rules));
        } catch (InstantiationException | IllegalAccessException e) {
            e.printStackTrace();
        }
    }

    public boolean has(String s) {
        return routes.containsKey(s) && routes.get(s) != null;
    }

    public Route get(String path) {
        return routes.get(path);
    }
}
