package krbzsq.surge.view;

import krbzsq.surge.Session;
import krbzsq.surge.net.HttpRequest;
import krbzsq.surge.net.HttpResponse;
import krbzsq.surge.view.style.Stylesheet;
import krbzsq.surge.view.template.DefaultTemplate;
import krbzsq.surge.view.template.Template;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public abstract class Page {
    private Map<Class<? extends Stylesheet>, Stylesheet> stylesheets = new HashMap<>();
    private String title = "Untitled";
    private Template template = new DefaultTemplate(this);

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public void addStylesheet(Stylesheet stylesheet) {
        stylesheets.put(stylesheet.getClass(), stylesheet.build());
    }

    public List<Stylesheet> getStylesheets() {
        return new ArrayList<>(stylesheets.values());
    }

    public Template getTemplate() {
        return template;
    }

    public void setTemplate(Template template) {
        this.template = template;
    }

    public String element(String tag, String content, String[]... strings) {
        StringBuilder bldr = new StringBuilder();
        bldr.append("<").append(tag);
        if(strings != null && strings.length > 0) {
            bldr.append(" ");
            for(String[] s : strings) {
                bldr.append(s[0]).append("=\"").append(s[1]).append("\"");
            }
        }
        bldr.append(">").append(content).append("</").append(tag).append(">\n");
        return bldr.toString();
    }

    public HttpResponse doRespond(int status, HttpRequest request, Session session) {
        StringBuilder bldr = new StringBuilder();
        bldr.append(template.getTop());
        bldr.append(respond(request, session));
        bldr.append(template.getBottom());
        return new HttpResponse(status, bldr.toString());
    }

    public abstract String respond(HttpRequest request, Session session);
}
