package krbzsq.surge.view.template;

import krbzsq.surge.view.Page;
import krbzsq.surge.view.style.Stylesheet;
import krbzsq.surge.view.style.StylesheetCompiler;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

public class TemplateLoader {

    public static Template load(File htmlFile, Page page) {
        try (BufferedReader reader = new BufferedReader(new FileReader(htmlFile))) {
            String line;
            final StringBuilder top = new StringBuilder();
            final StringBuilder bottom = new StringBuilder();
            boolean has = false;
            while ((line = reader.readLine()) != null) {
                line = line.trim();
                if(line.contains("#{TITLE}")) {
                    line = line.replace("#{TITLE}", page.getTitle());
                }
                if(line.equals("#{CONTENT}")) {
                    has = true;
                    continue;
                }
                if(line.equals("#{STYLESHEETS}")) {
                    if(page.getStylesheets().size() > 0) {
                        for(Stylesheet sheet : page.getStylesheets()) {
                            top.append("<style>\n");
                            top.append(StylesheetCompiler.compile(sheet)).append("\n");
                            top.append("</style>\n");
                        }
                    }
                    continue;
                }
                if(!has) {
                    top.append(line).append("\n");
                } else {
                    bottom.append(line).append("\n");
                }
            }
            return new Template() {
                @Override
                public String getTop() {
                    return top.toString();
                }

                @Override
                public String getBottom() {
                    return bottom.toString();
                }
            };
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }
}
